"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs = require("fs");
const path = require("path");
const Handlebars = require("handlebars");
const Shell = require("shelljs");
class Create extends command_1.Command {
    compileTemplate(data) {
        const { sourceFile, compiledFile, compilerOption } = data;
        const source = fs.readFileSync(Shell.pwd().toString() + `/${compilerOption.project_name}/${sourceFile}`, 'utf-8');
        const template = Handlebars.compile(source);
        const result = template(compilerOption);
        fs.writeFileSync(Shell.pwd().toString() + `/${compilerOption.project_name}/${compiledFile}`, result);
        Shell.rm(Shell.pwd().toString() + `/${compilerOption.project_name}/${sourceFile}`);
    }
    createFolder(project_name) {
        Shell.mkdir(project_name);
    }
    copyTemplate(project_name) {
        Shell.cp('-rf', path.join(__dirname, '../templates/create/*'), project_name);
        Shell.cp('-rf', path.join(__dirname, '../templates/create/.gitignore'), project_name);
        Shell.cp('-rf', path.join(__dirname, '../templates/create/.babelrc'), project_name);
    }
    installDependency(project_name) {
        Shell.cd(project_name);
        Shell.exec('npm i');
    }
    async run() {
        const { args } = this.parse(Create);
        const project_name = args.project_name;
        const fileToCompile = [
            {
                sourceFile: 'package.json.hbs',
                compiledFile: 'package.json',
                compilerOption: { 'project_name': project_name }
            },
            {
                sourceFile: 'src/components/App/App.tsx.hbs',
                compiledFile: 'src/components/App/App.tsx',
                compilerOption: { 'project_name': project_name }
            }
        ];
        if (project_name) {
            // create folder
            this.createFolder(project_name);
            // copy file
            this.copyTemplate(project_name);
            // compile template
            fileToCompile.map(data => this.compileTemplate(data));
            // install dependency
            this.installDependency(project_name);
        }
        else {
            await Create.run(['--help']);
        }
    }
}
exports.default = Create;
Create.description = 'create an empty starter project';
Create.examples = [
    `$ reactscript create new-app`,
];
Create.flags = {
    help: command_1.flags.help({ char: 'h' }),
};
Create.args = [
    {
        name: 'project_name'
    }
];
